@@ -1,14 +1,13 @@
 /* Evolution calendar - Alarm notification service object
  *
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  * Copyright (C) 2000 Ximian, Inc.
  *
  * Author: Federico Mena-Quintero <federico@ximian.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
 #ifndef ALARM_NOTIFY_H
 #define ALARM_NOTIFY_H
 
-#include <bonobo/bonobo-object.h>
+#include <bonobo/bonobo-xobject.h>
 #include "evolution-calendar.h"
 
 
@@ -41,26 +40,25 @@ typedef struct _AlarmNotifyClass AlarmNotifyClass;
 typedef struct _AlarmNotifyPrivate AlarmNotifyPrivate;
 
 struct _AlarmNotify {
-	BonoboObject object;
+	BonoboXObject xobject;
 
 	/* Private data */
 	AlarmNotifyPrivate *priv;
 };
 
 struct _AlarmNotifyClass {
-	BonoboObjectClass parent_class;
+	BonoboXObjectClass parent_class;
+
+	POA_GNOME_Evolution_Calendar_AlarmNotify__epv epv;
 };
 
 GtkType alarm_notify_get_type (void);
 
-AlarmNotify *alarm_notify_construct (AlarmNotify *an,
-				     GNOME_Evolution_Calendar_AlarmNotify corba_an);
-
-GNOME_Evolution_Calendar_AlarmNotify alarm_notify_corba_object_create (BonoboObject *object);
-POA_GNOME_Evolution_Calendar_AlarmNotify__epv *alarm_notify_get_epv (void);
-
 AlarmNotify *alarm_notify_new (void);
 
+void alarm_notify_add_calendar (AlarmNotify *an, const char *str_uri, gboolean load_afterwards,
+				CORBA_Environment *ev);
+
 
 
 
