@@ -1,12 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-msg-composer-attachment.c
  *
- * Copyright (C) 1999  Ximian, Inc.
+ * Copyright (C) 1999,2001  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * modify it under the terms of version 2 of the GNU General Public
  * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,6 +28,7 @@
    attachment manually. */
 
 #include <sys/stat.h>
+#include <errno.h>
 
 #include <gtk/gtknotebook.h>
 #include <gtk/gtktogglebutton.h>
@@ -63,7 +64,7 @@ destroy (GtkObject *object)
 	EMsgComposerAttachment *attachment;
 
 	attachment = E_MSG_COMPOSER_ATTACHMENT (object);
-
+	
 	camel_object_unref (CAMEL_OBJECT (attachment->body));
 	if (attachment->pixbuf_cache != NULL)
 		gdk_pixbuf_unref (attachment->pixbuf_cache);
@@ -143,12 +144,14 @@ e_msg_composer_attachment_get_type (void)
  * e_msg_composer_attachment_new:
  * @file_name: filename to attach
  * @disposition: Content-Disposition of the attachment
+ * @ex: exception
  *
  * Return value: the new attachment, or %NULL on error
  **/
 EMsgComposerAttachment *
-e_msg_composer_attachment_new (const gchar *file_name,
-			       const gchar *disposition)
+e_msg_composer_attachment_new (const char *file_name,
+			       const char *disposition,
+			       CamelException *ex)
 {
 	EMsgComposerAttachment *new;
 	CamelMimePart *part;
@@ -156,21 +159,33 @@ e_msg_composer_attachment_new (const gchar *file_name,
 	CamelStream *stream;
 	struct stat statbuf;
 	gchar *mime_type;
-	char *content_id;
 	char *filename;
 	
 	g_return_val_if_fail (file_name != NULL, NULL);
 	
-	if (stat (file_name, &statbuf) < 0)
+	if (stat (file_name, &statbuf) < 0) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot attach file %s: %s"),
+				      file_name, g_strerror (errno));
 		return NULL;
+	}
 	
 	/* return if it's not a regular file */
-	if (!S_ISREG (statbuf.st_mode))
+	if (!S_ISREG (statbuf.st_mode)) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot attach file %s: not a regular file"),
+				      file_name);
 		return NULL;
+	}
 	
 	stream = camel_stream_fs_new_with_name (file_name, O_RDONLY, 0);
-	if (!stream)
+	if (!stream) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot attach file %s: %s"),
+				      file_name, g_strerror (errno));
 		return NULL;
+	}
+	
 	wrapper = camel_data_wrapper_new ();
 	camel_data_wrapper_construct_from_stream (wrapper, stream);
 	camel_object_unref (CAMEL_OBJECT (stream));
@@ -193,7 +208,7 @@ e_msg_composer_attachment_new (const gchar *file_name,
 	
 	camel_mime_part_set_filename (part, filename);
 	g_free (filename);
-
+	
 #if 0
 	/* Note: Outlook 2002 is broken with respect to Content-Ids on
            non-multipart/related parts, so as an interoperability
@@ -206,6 +221,7 @@ e_msg_composer_attachment_new (const gchar *file_name,
 #endif
 	
 	new = e_msg_composer_attachment_new_from_mime_part (part);
+	camel_object_unref (CAMEL_OBJECT (part));
 	
 	new->size = statbuf.st_size;
 	new->guessed_type = TRUE;
@@ -308,49 +324,47 @@ connect_widget (GladeXML *gui,
 }
 
 static void
-close_cb (GtkWidget *widget,
-	  gpointer data)
+close_cb (GtkWidget *widget, gpointer data)
 {
 	EMsgComposerAttachment *attachment;
 	DialogData *dialog_data;
-
+	
 	dialog_data = (DialogData *) data;
 	attachment = dialog_data->attachment;
-
+	
 	gtk_widget_destroy (glade_xml_get_widget (attachment->editor_gui,
 						  "dialog"));
 	gtk_object_unref (GTK_OBJECT (attachment->editor_gui));
 	attachment->editor_gui = NULL;
-
+	
 	destroy_dialog_data (dialog_data);
 }
 
 static void
-ok_cb (GtkWidget *widget,
-       gpointer data)
+ok_cb (GtkWidget *widget, gpointer data)
 {
 	DialogData *dialog_data;
 	EMsgComposerAttachment *attachment;
-	gchar *str;
+	char *str;
 	
 	dialog_data = (DialogData *) data;
 	attachment = dialog_data->attachment;
-
+	
 	str = e_utf8_gtk_entry_get_text (dialog_data->file_name_entry);
 	camel_mime_part_set_filename (attachment->body, str);
 	g_free (str);
-
+	
 	str = e_utf8_gtk_entry_get_text (dialog_data->description_entry);
 	camel_mime_part_set_description (attachment->body, str);
 	g_free (str);
-
+	
 	str = e_utf8_gtk_entry_get_text (dialog_data->mime_type_entry);
 	camel_mime_part_set_content_type (attachment->body, str);
-
+	
 	camel_data_wrapper_set_mime_type (
 		camel_medium_get_content_object (CAMEL_MEDIUM (attachment->body)), str);
 	g_free (str);
-
+	
 	switch (gtk_toggle_button_get_active (dialog_data->disposition_checkbox)) {
 	case 0:
 		camel_mime_part_set_disposition (attachment->body, "attachment");
@@ -362,7 +376,7 @@ ok_cb (GtkWidget *widget,
 		/* Hmmmm? */
 		break;
 	}
-
+	
 	changed (attachment);
 	close_cb (widget, data);
 }
@@ -373,7 +387,7 @@ file_name_focus_out_cb (GtkWidget *widget,
 			gpointer data)
 {
 	DialogData *dialog_data;
-
+	
 	dialog_data = (DialogData *) data;
 	update_mime_type (dialog_data);
 }
@@ -385,32 +399,32 @@ e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment,
 {
 	DialogData *dialog_data;
 	GladeXML *editor_gui;
-
+	
 	g_return_if_fail (attachment != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_ATTACHMENT (attachment));
-
+	
 	if (attachment->editor_gui != NULL) {
 		GtkWidget *window;
-
+		
 		window = glade_xml_get_widget (attachment->editor_gui,
 					       "dialog");
 		gdk_window_show (window->window);
 		return;
 	}
-
+	
 	editor_gui = glade_xml_new (E_GLADEDIR "/e-msg-composer-attachment.glade",
 				    NULL);
 	if (editor_gui == NULL) {
 		g_warning ("Cannot load `e-msg-composer-attachment.glade'");
 		return;
 	}
-
+	
 	attachment->editor_gui = editor_gui;
-
+	
 	gtk_window_set_transient_for
 		(GTK_WINDOW (glade_xml_get_widget (editor_gui, "dialog")),
 		 GTK_WINDOW (gtk_widget_get_toplevel (parent)));
-
+	
 	dialog_data = g_new (DialogData, 1);
 	dialog_data->attachment = attachment;
 	dialog_data->dialog = glade_xml_get_widget (editor_gui, "dialog");
@@ -422,12 +436,12 @@ e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment,
 		glade_xml_get_widget (editor_gui, "mime_type_entry"));
 	dialog_data->disposition_checkbox = GTK_TOGGLE_BUTTON (
 		glade_xml_get_widget (editor_gui, "disposition_checkbox"));
-
+	
 	if (attachment != NULL) {
 		CamelContentType *content_type;
 		char *type;
 		const char *disposition;
-
+		
 		set_entry (editor_gui, "file_name_entry",
 			   camel_mime_part_get_filename (attachment->body));
 		set_entry (editor_gui, "description_entry",
@@ -436,15 +450,19 @@ e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment,
 		type = header_content_type_simple (content_type);
 		set_entry (editor_gui, "mime_type_entry", type);
 		g_free (type);
-
+		
 		disposition = camel_mime_part_get_disposition (attachment->body);
 		gtk_toggle_button_set_active (dialog_data->disposition_checkbox,
 					      disposition && !g_strcasecmp (disposition, "inline"));
 	}
-
+	
 	connect_widget (editor_gui, "ok_button", "clicked", ok_cb, dialog_data);
 	connect_widget (editor_gui, "close_button", "clicked", close_cb, dialog_data);
-
+	
 	connect_widget (editor_gui, "file_name_entry", "focus_out_event",
 			file_name_focus_out_cb, dialog_data);
+	
+	gtk_signal_connect (GTK_OBJECT (parent), "destroy", close_cb, dialog_data);
+	gtk_signal_connect (GTK_OBJECT (gtk_widget_get_toplevel (parent)), "hide",
+			    close_cb, dialog_data);
 }
