@@ -28,12 +28,12 @@
 #include <gal/e-table/e-cell-size.h>
 #include <gal/e-table/e-tree-memory.h>
 #include <gal/e-table/e-tree-memory-callbacks.h>
+#include <gal/unicode/gunicode.h>
 
 #include <camel/camel-exception.h>
 #include <camel/camel-file-utils.h>
 #include <camel/camel-folder.h>
 #include <camel/camel-folder-thread.h>
-#include <camel/camel-vtrash-folder.h>
 #include <e-util/ename/e-name-western.h>
 #include <e-util/e-memory.h>
 
@@ -229,7 +229,7 @@ e_mail_address_compare (gconstpointer address1, gconstpointer address2)
 	if (!addr2->wname->last)
 		return 1;
 	
-	retval = g_strcasecmp (addr1->wname->last, addr2->wname->last);
+	retval = g_utf8_collate (addr1->wname->last, addr2->wname->last);
 	if (retval) 
 		return retval;
 	
@@ -243,7 +243,7 @@ e_mail_address_compare (gconstpointer address1, gconstpointer address2)
 	if (!addr2->wname->first)
 		return 1;
 	
-	retval = g_strcasecmp (addr1->wname->first, addr2->wname->first);
+	retval = g_utf8_collate (addr1->wname->first, addr2->wname->first);
 	if (retval) 
 		return retval;
 	
@@ -257,7 +257,7 @@ address_compare (gconstpointer address1, gconstpointer address2)
 #ifdef SMART_ADDRESS_COMPARE
 	EMailAddress *addr1, *addr2;
 #endif /* SMART_ADDRESS_COMPARE */
-	gint retval;
+	int retval;
 	
 	g_return_val_if_fail (address1 != NULL, 1);
 	g_return_val_if_fail (address2 != NULL, -1);
@@ -269,7 +269,7 @@ address_compare (gconstpointer address1, gconstpointer address2)
 	e_mail_address_free (addr1);
 	e_mail_address_free (addr2);
 #else
-	retval = g_strcasecmp ((const char *) address1, (const char *) address2);
+	retval = g_utf8_collate ((const char *) address1, (const char *) address2);
 #endif /* SMART_ADDRESS_COMPARE */
 	
 	return retval;
@@ -289,22 +289,25 @@ subject_compare (gconstpointer subject1, gconstpointer subject2)
 	while (!g_strncasecmp (sub1, "Re:", 3)) {
 		sub1 += 3;
 		/* jump over any spaces */
-		for ( ; *sub1 && isspace (*sub1); sub1++);
+		while (*sub1 && g_unichar_isspace (g_utf8_get_char (sub1)))
+			sub1 = g_utf8_next_char (sub1);
 	}
 	
 	/* trim off any "Re:"'s at the beginning of subject2 */
 	sub2 = (char *) subject2;
 	while (!g_strncasecmp (sub2, "Re:", 3)) {
 		sub2 += 3;
-		/* jump over any spaces */
-		for ( ; *sub2 && isspace (*sub2); sub2++);
+		while (*sub2 && g_unichar_isspace (g_utf8_get_char (sub2)))
+			sub2 = g_utf8_next_char (sub2);
 	}
 	
 	/* jump over any spaces */
-	for ( ; *sub1 && isspace (*sub1); sub1++);
-	for ( ; *sub2 && isspace (*sub2); sub2++);
+	while (*sub1 && g_unichar_isspace (g_utf8_get_char (sub1)))
+		sub1 = g_utf8_next_char (sub1);
+	while (*sub2 && g_unichar_isspace (g_utf8_get_char (sub2)))
+		sub2 = g_utf8_next_char (sub2);
 	
-	return g_strcasecmp (sub1, sub2);
+	return g_utf8_collate (sub1, sub2);
 }
 
 static gchar *
@@ -330,11 +333,13 @@ static const char *
 get_message_uid (MessageList *message_list, ETreePath node)
 {
 	CamelMessageInfo *info;
-
-	g_return_val_if_fail (node != NULL, NULL);
-	info = e_tree_memory_node_get_data (E_TREE_MEMORY(message_list->model), node);
-
-	return camel_message_info_uid(info);
+	
+	g_assert (node != NULL);
+	info = e_tree_memory_node_get_data (E_TREE_MEMORY (message_list->model), node);
+	/* correct me if I'm wrong, but this should never be NULL, should it? */
+	g_assert (info != NULL);
+	
+	return camel_message_info_uid (info);
 }
 
 /* Gets the CamelMessageInfo for the message displayed at the given
@@ -343,8 +348,13 @@ get_message_uid (MessageList *message_list, ETreePath node)
 static CamelMessageInfo *
 get_message_info (MessageList *message_list, ETreePath node)
 {
-	g_return_val_if_fail (node != NULL, NULL);
-	return e_tree_memory_node_get_data (E_TREE_MEMORY (message_list->model), node);
+	CamelMessageInfo *info;
+	
+	g_assert (node != NULL);
+	info = e_tree_memory_node_get_data (E_TREE_MEMORY (message_list->model), node);
+	g_assert (info != NULL);
+	
+	return info;
 }
 
 /**
@@ -354,6 +364,8 @@ get_message_info (MessageList *message_list, ETreePath node)
  * @direction: the direction to search in
  * @flags: a set of flag values
  * @mask: a mask for comparing against @flags
+ * @wraparound: if %TRUE, go back to the beginning for 
+ *     the next match if necessary.
  *
  * This moves the message list selection to a suitable row. @base_row
  * lists the first (model) row to try, but as a special case, model
@@ -365,17 +377,19 @@ get_message_info (MessageList *message_list, ETreePath node)
  * unchanged.
  **/
 void
-message_list_select (MessageList *message_list, int base_row,
+message_list_select (MessageList               *message_list,
+		     int                        base_row,
 		     MessageListSelectDirection direction,
-		     guint32 flags, guint32 mask)
+		     guint32                    flags,
+		     guint32                    mask,
+		     gboolean                   wraparound)
 {
 	CamelMessageInfo *info;
 	int vrow, last;
-	ETree *et = message_list->tree;
 	
 	if (!GTK_WIDGET_HAS_FOCUS (message_list))
 		gtk_widget_grab_focus (GTK_WIDGET (message_list));
-
+	
 	switch (direction) {
 	case MESSAGE_LIST_SELECT_PREVIOUS:
 		last = -1;
@@ -389,26 +403,29 @@ message_list_select (MessageList *message_list, int base_row,
 		g_warning("Invalid argument to message_list_select");
 		return;
 	}
-
+	
 	/* If it's -1, we want the last view row, not the last model row. */
 	/* model_to_view_row etc simply dont work for sorted views.  Sigh. */
 	if (base_row == -1)
-		vrow = e_tree_row_count(message_list->tree) - 1;
+		vrow = e_tree_row_count (message_list->tree) - 1;
 	else
-		vrow = e_tree_model_to_view_row (et, base_row);
-
-	if (base_row <= -1)
+		vrow = e_tree_model_to_view_row (message_list->tree, base_row);
+	
+	if (vrow <= -1)
 		return;
+	
 	/* This means that we'll move at least one message in 'direction'. */
 	if (vrow != last)
 		vrow += direction;
-
+	
 	/* We don't know whether to use < or > due to "direction" */
 	while (vrow != last) {
-		ETreePath node = e_tree_node_at_row(et, vrow);
+		ETreePath node = e_tree_node_at_row (message_list->tree, vrow);
+		
 		info = get_message_info (message_list, node);
+		
 		if (info && (info->flags & mask) == flags) {
-			e_tree_set_cursor (et, node);
+			e_tree_set_cursor (message_list->tree, node);
 			
 			gtk_signal_emit (GTK_OBJECT (message_list), message_list_signals[MESSAGE_SELECTED],
 					 camel_message_info_uid (info));
@@ -416,6 +433,15 @@ message_list_select (MessageList *message_list, int base_row,
 		}
 		vrow += direction;
 	}
+	
+	if (wraparound) {
+		if (direction == MESSAGE_LIST_SELECT_PREVIOUS)
+			base_row = -1;
+		else
+			base_row = 0;
+		
+		message_list_select (message_list, base_row, direction, flags, mask, FALSE);
+	}
 }
 
 
@@ -495,7 +521,7 @@ ml_has_get_node_by_id (ETreeModel *etm, void *data)
  * SimpleTableModel::get_save_id
  */
 static ETreePath
-ml_get_node_by_id (ETreeModel *etm, char *save_id, void *data)
+ml_get_node_by_id (ETreeModel *etm, const char *save_id, void *data)
 {
 	MessageList *ml;
 
@@ -813,7 +839,7 @@ ml_tree_value_at (ETreeModel *etm, ETreePath path, int col, void *model_data)
 		child = e_tree_model_node_get_first_child(etm, path);
 		if (child && !e_tree_node_is_expanded(message_list->tree, path)
 		    && (msg_info->flags & CAMEL_MESSAGE_SEEN)) {
-			return (void *)subtree_unread(message_list, child);
+			return GINT_TO_POINTER (subtree_unread (message_list, child));
 		}
 
 		return GINT_TO_POINTER (!(msg_info->flags & CAMEL_MESSAGE_SEEN));
@@ -881,7 +907,7 @@ filter_date (time_t date)
 	if (then.tm_mday == now.tm_mday &&
 	    then.tm_mon == now.tm_mon &&
 	    then.tm_year == now.tm_year) {
-		strftime (buf, 26, _("Today %l:%M %p"), &then);
+		e_strftime_fix_am_pm (buf, 26, _("Today %l:%M %p"), &then);
 		done = TRUE;
 	}
 	if (!done) {
@@ -890,7 +916,7 @@ filter_date (time_t date)
 		if (then.tm_mday == yesterday.tm_mday &&
 		    then.tm_mon == yesterday.tm_mon &&
 		    then.tm_year == yesterday.tm_year) {
-			strftime (buf, 26, _("Yesterday %l:%M %p"), &then);
+			e_strftime_fix_am_pm (buf, 26, _("Yesterday %l:%M %p"), &then);
 			done = TRUE;
 		}
 	}
@@ -902,7 +928,7 @@ filter_date (time_t date)
 			if (then.tm_mday == yesterday.tm_mday &&
 			    then.tm_mon == yesterday.tm_mon &&
 			    then.tm_year == yesterday.tm_year) {
-				strftime (buf, 26, _("%a %l:%M %p"), &then);
+				e_strftime_fix_am_pm (buf, 26, _("%a %l:%M %p"), &then);
 				done = TRUE;
 				break;
 			}
@@ -910,9 +936,9 @@ filter_date (time_t date)
 	}
 	if (!done) {
 		if (then.tm_year == now.tm_year) {
-			strftime (buf, 26, _("%b %d %l:%M %p"), &then);
+			e_strftime_fix_am_pm (buf, 26, _("%b %d %l:%M %p"), &then);
 		} else {
-			strftime (buf, 26, _("%b %d %Y"), &then);
+			e_strftime_fix_am_pm (buf, 26, _("%b %d %Y"), &then);
 		}
 	}
 #if 0
@@ -1016,7 +1042,7 @@ sort_info_changed (GtkWidget *widget, MessageList *ml)
 }
 
 static void
-message_list_setup_etree(MessageList *message_list, gboolean outgoing)
+message_list_setup_etree (MessageList *message_list, gboolean outgoing)
 {
 	ETableState *etstate;
 	
@@ -1026,10 +1052,14 @@ message_list_setup_etree(MessageList *message_list, gboolean outgoing)
 		char *path;
 		char *name;
 		struct stat st;
+
+		gtk_object_set (GTK_OBJECT (message_list->tree),
+				"uniform_row_height", TRUE,
+				NULL);
 		
 		name = camel_service_get_name (CAMEL_SERVICE (message_list->folder->parent_store), TRUE);
-		printf ("folder name is '%s'\n", name);
-
+		d(printf ("folder name is '%s'\n", name));
+		
 		path = mail_config_folder_to_cachename (message_list->folder, "et-header-");
 		if (path && stat (path, &st) == 0 && st.st_size > 0 && S_ISREG (st.st_mode)) {
 			/* build based on saved file */
@@ -1051,20 +1081,20 @@ message_list_setup_etree(MessageList *message_list, gboolean outgoing)
 			e_tree_load_expanded_state (message_list->tree, path);
 		}
 		g_free (path);
-
-		g_free (name);
-
-		etstate = e_tree_get_state_object(message_list->tree);
-		gtk_signal_connect(GTK_OBJECT(etstate->sort_info), 
-				   "sort_info_changed",
-				   GTK_SIGNAL_FUNC(sort_info_changed),
-				   message_list);
-		gtk_signal_connect(GTK_OBJECT(etstate->sort_info), 
-				   "group_info_changed",
-				   GTK_SIGNAL_FUNC(sort_info_changed),
-				   message_list);
-
 		
+		g_free (name);
+		
+		etstate = e_tree_get_state_object (message_list->tree);
+		gtk_signal_connect (GTK_OBJECT (etstate->sort_info), 
+				    "sort_info_changed",
+				    GTK_SIGNAL_FUNC (sort_info_changed),
+				    message_list);
+		gtk_signal_connect (GTK_OBJECT (etstate->sort_info), 
+				    "group_info_changed",
+				    GTK_SIGNAL_FUNC (sort_info_changed),
+				    message_list);
+		
+		gtk_object_unref (GTK_OBJECT (etstate));
 	}
 }
 
@@ -1077,7 +1107,8 @@ message_list_init (GtkObject *object)
 	MessageList *message_list = MESSAGE_LIST (object);
 
 	e_scroll_frame_set_policy (E_SCROLL_FRAME (message_list),
-				   GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+				   GTK_POLICY_NEVER,
+				   GTK_POLICY_ALWAYS);
 
 	message_list->hidden = NULL;
 	message_list->hidden_pool = NULL;
@@ -1089,6 +1120,7 @@ message_list_init (GtkObject *object)
 	message_list->hide_lock = g_mutex_new();
 
 	message_list->uid_nodemap = g_hash_table_new (g_str_hash, g_str_equal);
+	message_list->async_event = mail_async_event_new();
 }
 
 static void
@@ -1096,20 +1128,12 @@ message_list_destroy (GtkObject *object)
 {
 	MessageList *message_list = MESSAGE_LIST (object);
 
+	mail_async_event_destroy(message_list->async_event);
+
 	if (message_list->folder) {
 		save_tree_state(message_list);
 		hide_save_state(message_list);
-	}
 
-	gtk_object_unref (GTK_OBJECT (message_list->model));
-
-	if (message_list->idle_id != 0)
-		g_source_remove(message_list->idle_id);
-	
-	if (message_list->seen_id)
-		gtk_timeout_remove (message_list->seen_id);
-	
-	if (message_list->folder) {
 		camel_object_unhook_event((CamelObject *)message_list->folder, "folder_changed",
 					  folder_changed, message_list);
 		camel_object_unhook_event((CamelObject *)message_list->folder, "message_changed",
@@ -1117,6 +1141,15 @@ message_list_destroy (GtkObject *object)
 		camel_object_unref (CAMEL_OBJECT (message_list->folder));
 	}
 
+	gtk_object_unref (GTK_OBJECT (message_list->extras));
+	gtk_object_unref (GTK_OBJECT (message_list->model));
+
+	if (message_list->idle_id != 0)
+		g_source_remove (message_list->idle_id);
+	
+	if (message_list->seen_id)
+		gtk_timeout_remove (message_list->seen_id);
+
 	if (message_list->hidden) {
 		g_hash_table_destroy(message_list->hidden);
 		e_mempool_destroy(message_list->hidden_pool);
@@ -1209,7 +1242,7 @@ message_list_construct (MessageList *message_list)
 	
 	message_list->tree = e_tree_scrolled_get_tree(E_TREE_SCROLLED (message_list));
 	e_tree_root_node_set_visible (message_list->tree, FALSE);
-	
+
 	gtk_signal_connect (GTK_OBJECT (message_list->tree), "cursor_activated",
 			    GTK_SIGNAL_FUNC (on_cursor_activated_cmd),
 			    message_list);
@@ -1256,9 +1289,9 @@ clear_tree (MessageList *ml)
 
 	/* we also reset the uid_rowmap since it is no longer useful/valid anyway */
 	if (ml->folder)
-		g_hash_table_foreach(ml->uid_nodemap, (GHFunc)clear_info, ml);
+		g_hash_table_foreach (ml->uid_nodemap, (GHFunc)clear_info, ml);
 	g_hash_table_destroy (ml->uid_nodemap);
-	ml->uid_nodemap = g_hash_table_new(g_str_hash, g_str_equal);
+	ml->uid_nodemap = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	if (ml->tree_root) {
 		/* we should be frozen already */
@@ -1279,18 +1312,19 @@ clear_tree (MessageList *ml)
 /* we try and find something that isn't deleted in our tree
    there is actually no assurance that we'll find somethign that will
    still be there next time, but its probably going to work most of the time */
-static char *find_next_undeleted(MessageList *ml)
+static char *
+find_next_undeleted (MessageList *ml)
 {
 	ETreePath node;
 	int last;
 	int vrow;
 	ETree *et = ml->tree;
 	CamelMessageInfo *info;
-
-	node = g_hash_table_lookup(ml->uid_nodemap, ml->cursor_uid);
+	
+	node = g_hash_table_lookup (ml->uid_nodemap, ml->cursor_uid);
 	if (node == NULL)
 		return NULL;
-
+	
 	info = get_message_info (ml, node);
 	if (info && (info->flags & CAMEL_MESSAGE_DELETED) == 0) {
 		return NULL;
@@ -1310,7 +1344,7 @@ static char *find_next_undeleted(MessageList *ml)
 		node = e_tree_node_at_row (et, vrow);
 		info = get_message_info (ml, node);
 		if (info && (info->flags & CAMEL_MESSAGE_DELETED) == 0) {
-			return g_strdup (camel_message_info_uid(info));
+			return g_strdup (camel_message_info_uid (info));
 		}
 		vrow ++;
 	}
@@ -1732,7 +1766,7 @@ build_flat_diff(MessageList *ml, CamelFolderChangeInfo *changes)
 	gettimeofday(&start, NULL);
 #endif
 
-	printf("updating changes to display\n");
+	d(printf("updating changes to display\n"));
 
 	/* remove individual nodes? */
 	d(printf("Removing messages from view:\n"));
@@ -1749,20 +1783,22 @@ build_flat_diff(MessageList *ml, CamelFolderChangeInfo *changes)
 	/* add new nodes? - just append to the end */
 	d(printf("Adding messages to view:\n"));
 	for (i=0;i<changes->uid_added->len;i++) {
-		info = camel_folder_get_message_info(ml->folder, changes->uid_added->pdata[i]);
+		info = camel_folder_get_message_info (ml->folder, changes->uid_added->pdata[i]);
 		if (info) {
 			d(printf(" %s\n", (char *)changes->uid_added->pdata[i]));
-			node = e_tree_memory_node_insert(E_TREE_MEMORY(ml->model), ml->tree_root, -1, info);
-			g_hash_table_insert(ml->uid_nodemap, (void *)camel_message_info_uid(info), node);
+			node = e_tree_memory_node_insert (E_TREE_MEMORY (ml->model), ml->tree_root, -1, info);
+			g_hash_table_insert (ml->uid_nodemap, (void *)camel_message_info_uid (info), node);
 		}
 	}
 
 	/* and update changes too */
 	d(printf("Changing messages to view:\n"));
-	for (i=0;i<changes->uid_changed->len;i++) {
-		ETreePath *node = g_hash_table_lookup(ml->uid_nodemap, changes->uid_changed->pdata[i]);
-		if (node)
-			e_tree_model_node_data_changed(ml->model, node);
+	for (i = 0; i < changes->uid_changed->len; i++) {
+		ETreePath *node = g_hash_table_lookup (ml->uid_nodemap, changes->uid_changed->pdata[i]);
+		if (node) {
+			e_tree_model_pre_change (ml->model);
+			e_tree_model_node_data_changed (ml->model, node);
+		}
 	}
 
 #ifdef TIMEIT
@@ -1783,63 +1819,64 @@ main_folder_changed (CamelObject *o, gpointer event_data, gpointer user_data)
 	CamelMessageInfo *info;
 	CamelFolder *folder = (CamelFolder *)o;
 	int i;
-
-	printf("folder changed event, changes = %p\n", changes);
+	
+	d(printf("folder changed event, changes = %p\n", changes));
 	if (changes) {
-		printf("changed = %d added = %d removed = %d\n",
-		       changes->uid_changed->len, changes->uid_added->len, changes->uid_removed->len);
-
+		d(printf("changed = %d added = %d removed = %d\n",
+			 changes->uid_changed->len, changes->uid_added->len, changes->uid_removed->len));
+		
 		/* check if the hidden state has changed, if so modify accordingly, then regenerate */
 		if (ml->hidedeleted) {
-			newchanges = camel_folder_change_info_new();
+			newchanges = camel_folder_change_info_new ();
 			
-			for (i=0;i<changes->uid_changed->len;i++) {
+			for (i = 0; i < changes->uid_changed->len; i++) {
 				ETreePath node = g_hash_table_lookup (ml->uid_nodemap, changes->uid_changed->pdata[i]);
-
-				info = camel_folder_get_message_info(folder, changes->uid_changed->pdata[i]);
+				
+				info = camel_folder_get_message_info (folder, changes->uid_changed->pdata[i]);
 				if (node != NULL && info != NULL && (info->flags & CAMEL_MESSAGE_DELETED) != 0) {
-					camel_folder_change_info_remove_uid(newchanges, changes->uid_changed->pdata[i]);
+					camel_folder_change_info_remove_uid (newchanges, changes->uid_changed->pdata[i]);
 				} else if (node == NULL && info != NULL && (info->flags & CAMEL_MESSAGE_DELETED) == 0) {
-					camel_folder_change_info_add_uid(newchanges, changes->uid_changed->pdata[i]);
+					camel_folder_change_info_add_uid (newchanges, changes->uid_changed->pdata[i]);
 				} else {
-					camel_folder_change_info_change_uid(newchanges, changes->uid_changed->pdata[i]);
+					camel_folder_change_info_change_uid (newchanges, changes->uid_changed->pdata[i]);
 				}
-				camel_folder_free_message_info(folder, info);
+				camel_folder_free_message_info (folder, info);
 			}
 			
 			if (newchanges->uid_added->len > 0 || newchanges->uid_removed->len > 0) {
-				for (i=0;i<changes->uid_added->len;i++)
-					camel_folder_change_info_add_uid(newchanges, changes->uid_added->pdata[i]);
-				for (i=0;i<changes->uid_removed->len;i++)
-					camel_folder_change_info_remove_uid(newchanges, changes->uid_removed->pdata[i]);
-				camel_folder_change_info_free(changes);
+				for (i = 0; i < changes->uid_added->len; i++)
+					camel_folder_change_info_add_uid (newchanges, changes->uid_added->pdata[i]);
+				for (i = 0; i < changes->uid_removed->len; i++)
+					camel_folder_change_info_remove_uid (newchanges, changes->uid_removed->pdata[i]);
+				camel_folder_change_info_free (changes);
 				changes = newchanges;
 			} else {
-				camel_folder_change_info_free(newchanges);
+				camel_folder_change_info_free (newchanges);
 			}
 		}
-
+		
 		if (changes->uid_added->len == 0 && changes->uid_removed->len == 0 && changes->uid_changed->len < 100) {
-			for (i=0;i<changes->uid_changed->len;i++) {
+			for (i = 0; i < changes->uid_changed->len; i++) {
 				ETreePath node = g_hash_table_lookup (ml->uid_nodemap, changes->uid_changed->pdata[i]);
-				if (node)
-					e_tree_model_node_data_changed(ml->model, node);
+				if (node) {
+					e_tree_model_pre_change (ml->model);
+					e_tree_model_node_data_changed (ml->model, node);
+				}
 			}
-
-			camel_folder_change_info_free(changes);
+			
+			camel_folder_change_info_free (changes);
 			return;
 		}
 	}
-
-	mail_regen_list(ml, ml->search, NULL, changes);
+	
+	mail_regen_list (ml, ml->search, NULL, changes);
 }
 
 static void
 folder_changed (CamelObject *o, gpointer event_data, gpointer user_data)
 {
-	/* similarly to message_changed, copy the change list and propagate it to
-	   the main thread and free it */
 	CamelFolderChangeInfo *changes;
+	MessageList *ml = MESSAGE_LIST (user_data);
 
 	if (event_data) {
 		changes = camel_folder_change_info_new();
@@ -1847,29 +1884,20 @@ folder_changed (CamelObject *o, gpointer event_data, gpointer user_data)
 	} else {
 		changes = NULL;
 	}
-	mail_proxy_event (main_folder_changed, o, changes, user_data);
-}
 
-static void
-main_message_changed (CamelObject *o, gpointer uid, gpointer user_data)
-{
-	MessageList *ml = MESSAGE_LIST (user_data);
-	CamelFolderChangeInfo *changes;
-
-	changes = camel_folder_change_info_new();
-	camel_folder_change_info_change_uid(changes, uid);
-	main_folder_changed(o, changes, ml);
-	g_free(uid);
+	mail_async_event_emit(ml->async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)main_folder_changed, o, changes, user_data);
 }
 
 static void
 message_changed (CamelObject *o, gpointer event_data, gpointer user_data)
 {
-	/* Here we copy the data because our thread may free the copy that we would reference.
-	 * The other thread would be passed a uid parameter that pointed to freed data.
-	 * We copy it and free it in the handler. 
-	 */
-	mail_proxy_event (main_message_changed, o, g_strdup ((gchar *)event_data), user_data);
+	CamelFolderChangeInfo *changes;
+	MessageList *ml = MESSAGE_LIST (user_data);
+
+	changes = camel_folder_change_info_new();
+	camel_folder_change_info_change_uid(changes, (char *)event_data);
+
+	mail_async_event_emit(ml->async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)main_folder_changed, o, changes, user_data);
 }
 
 void
@@ -1905,8 +1933,8 @@ message_list_set_folder (MessageList *message_list, CamelFolder *camel_folder, g
 	}
 
 	if (camel_folder) {
-		/* Setup the strikeout effect for non-vtrash folders */
-		if (!CAMEL_IS_VTRASH_FOLDER (camel_folder)) {
+		/* Setup the strikeout effect for non-trash folders */
+		if (!(camel_folder->folder_flags & CAMEL_FOLDER_IS_TRASH)) {
 			ECell *cell;
 
 			cell = e_table_extras_get_cell (message_list->extras, "render_date");
@@ -1925,9 +1953,6 @@ message_list_set_folder (MessageList *message_list, CamelFolder *camel_folder, g
 					NULL);
 		}
 
-		/* Now we're finally done with the extras */
-		gtk_object_sink (GTK_OBJECT (message_list->extras));
-
 		/* Build the etree suitable for this folder */
 		message_list_setup_etree (message_list, outgoing);
 		
@@ -1939,7 +1964,7 @@ message_list_set_folder (MessageList *message_list, CamelFolder *camel_folder, g
 		camel_object_ref (CAMEL_OBJECT (camel_folder));
 		
 		message_list->hidedeleted = mail_config_get_hide_deleted () &&
-			!(CAMEL_IS_VTRASH_FOLDER (camel_folder));
+			!(camel_folder->folder_flags & CAMEL_FOLDER_IS_TRASH);
 		
 		hide_load_state (message_list);
 		mail_regen_list (message_list, message_list->search, NULL, NULL);
@@ -1952,10 +1977,18 @@ static gboolean
 on_cursor_activated_idle (gpointer data)
 {
 	MessageList *message_list = data;
-
-	printf("emitting cursor changed signal, for uid %s\n", message_list->cursor_uid);
-	gtk_signal_emit(GTK_OBJECT (message_list), message_list_signals[MESSAGE_SELECTED], message_list->cursor_uid);
-
+	ESelectionModel *esm = e_tree_get_selection_model (message_list->tree);
+	gint selected = e_selection_model_selected_count (esm);
+	
+	if (selected == 1 && message_list->cursor_uid) {
+		d(printf ("emitting cursor changed signal, for uid %s\n", message_list->cursor_uid));
+		gtk_signal_emit (GTK_OBJECT (message_list),
+				 message_list_signals[MESSAGE_SELECTED], message_list->cursor_uid);
+	} else {
+		gtk_signal_emit (GTK_OBJECT (message_list),
+				 message_list_signals[MESSAGE_SELECTED], NULL);
+	}
+		
 	message_list->idle_id = 0;
 	return FALSE;
 }
@@ -1963,23 +1996,21 @@ on_cursor_activated_idle (gpointer data)
 static void
 on_cursor_activated_cmd (ETree *tree, int row, ETreePath path, gpointer user_data)
 {
-	MessageList *message_list;
+	MessageList *message_list = MESSAGE_LIST (user_data);
 	const char *new_uid;
 	
-	message_list = MESSAGE_LIST (user_data);
-
 	if (path == NULL)
 		new_uid = NULL;
 	else
-		new_uid = get_message_uid(message_list, path);
-
+		new_uid = get_message_uid (message_list, path);
+	
 	if (message_list->cursor_uid != NULL && new_uid != NULL && !strcmp (message_list->cursor_uid, new_uid))
 		return;
-
+	
 	message_list->cursor_row = row;
-	g_free(message_list->cursor_uid);
+	g_free (message_list->cursor_uid);
 	message_list->cursor_uid = g_strdup (new_uid);
-
+	
 	if (!message_list->idle_id) {
 		message_list->idle_id =
 			g_idle_add_full (G_PRIORITY_LOW, on_cursor_activated_idle,
@@ -1999,23 +2030,23 @@ on_click (ETree *tree, gint row, ETreePath path, gint col, GdkEvent *event, Mess
 		flag = CAMEL_MESSAGE_FLAGGED;
 	else
 		return FALSE;
-
-	info = get_message_info(list, path);
+	
+	info = get_message_info (list, path);
 	if (info == NULL) {
 		return FALSE;
 	}
-
+	
 	/* If a message was marked as deleted and the user flags it as important, undelete it */
 	if (col == COL_FLAGGED && (info->flags & CAMEL_MESSAGE_DELETED))
 		flag |= CAMEL_MESSAGE_DELETED;
-
-	camel_folder_set_message_flags(list->folder, camel_message_info_uid(info), flag, ~info->flags);
-
+	
+	camel_folder_set_message_flags (list->folder, camel_message_info_uid (info), flag, ~info->flags);
+	
 	if (flag == CAMEL_MESSAGE_SEEN && list->seen_id) {
 		gtk_timeout_remove (list->seen_id);
 		list->seen_id = 0;
 	}
-
+	
 	return TRUE;
 }
 
@@ -2026,16 +2057,23 @@ struct message_list_foreach_data {
 };
 
 static void
-mlfe_callback (int row, gpointer user_data)
+mlfe_callback (ETreePath path, gpointer user_data)
 {
 	struct message_list_foreach_data *mlfe_data = user_data;
 	const char *uid;
 
-	uid = get_message_uid (mlfe_data->message_list, e_tree_node_at_row(mlfe_data->message_list->tree, row));
+	if (e_tree_model_node_is_root (mlfe_data->message_list->model, path))
+		return;
+	
+	uid = get_message_uid (mlfe_data->message_list,
+			       path);
 	if (uid) {
-		mlfe_data->callback (mlfe_data->message_list,
-				     uid,
+		mlfe_data->callback (mlfe_data->message_list, uid,
 				     mlfe_data->user_data);
+	} else {
+		/* FIXME: could this the cause of bug #6637 and friends? */
+		g_warning ("I wonder if this could be the cause of bug #6637 and friends?");
+		g_assert_not_reached ();
 	}
 }
 
@@ -2045,43 +2083,40 @@ message_list_foreach (MessageList *message_list,
 		      gpointer user_data)
 {
 	struct message_list_foreach_data mlfe_data;
-
+	
 	mlfe_data.message_list = message_list;
 	mlfe_data.callback = callback;
 	mlfe_data.user_data = user_data;
-	e_tree_selected_row_foreach (message_list->tree,
-				     mlfe_callback, &mlfe_data);
+	e_tree_selected_path_foreach (message_list->tree,
+				      mlfe_callback, &mlfe_data);
 }
 
 /* set whether we are in threaded view or flat view */
 void
-message_list_set_threaded(MessageList *ml, gboolean threaded)
+message_list_set_threaded (MessageList *ml, gboolean threaded)
 {
-	if (ml->threaded ^ threaded) {
+	if (ml->threaded != threaded) {
 		ml->threaded = threaded;
-
-		mail_regen_list(ml, ml->search, NULL, NULL);
+		
+		mail_regen_list (ml, ml->search, NULL, NULL);
 	}
 }
 
 void
-message_list_set_hidedeleted(MessageList *ml, gboolean hidedeleted)
+message_list_set_hidedeleted (MessageList *ml, gboolean hidedeleted)
 {
-	if (ml->folder && CAMEL_IS_VTRASH_FOLDER(ml->folder))
-		hidedeleted = FALSE;
-
-	if (ml->hidedeleted ^ hidedeleted) {
+	if (ml->hidedeleted != hidedeleted) {
 		ml->hidedeleted = hidedeleted;
-
-		mail_regen_list(ml, ml->search, NULL, NULL);
+		
+		mail_regen_list (ml, ml->search, NULL, NULL);
 	}
 }
 
 void
-message_list_set_search(MessageList *ml, const char *search)
+message_list_set_search (MessageList *ml, const char *search)
 {
 	if (search == NULL || search[0] == '\0')
-		if (ml->search == NULL || ml->search[0]=='\0')
+		if (ml->search == NULL || ml->search[0] == '\0')
 			return;
 	
 	if (search != NULL && ml->search != NULL && strcmp (search, ml->search) == 0)
@@ -2092,11 +2127,26 @@ message_list_set_search(MessageList *ml, const char *search)
 
 /* returns the number of messages displayable *after* expression hiding has taken place */
 unsigned int
-message_list_length(MessageList *ml)
+message_list_length (MessageList *ml)
 {
 	return ml->hide_unhidden;
 }
 
+/* returns number of hidden messages */
+unsigned int
+message_list_hidden(MessageList *ml)
+{
+	unsigned int hidden = 0;
+
+	MESSAGE_LIST_LOCK (ml, hide_lock);
+	if (ml->hidden)
+		hidden = g_hash_table_size (ml->hidden);
+	MESSAGE_LIST_UNLOCK (ml, hide_lock);
+
+	return hidden;
+}
+
+
 /* add a new expression to hide, or set the range.
    @expr: A new search expression - all matching messages will be hidden.  May be %NULL.
    @lower: Use ML_HIDE_NONE_START to specify no messages hidden from the start of the list.
@@ -2111,46 +2161,46 @@ message_list_length(MessageList *ml)
    100 messages.
 */
 void
-message_list_hide_add(MessageList *ml, const char *expr, unsigned int lower, unsigned int upper)
+message_list_hide_add (MessageList *ml, const char *expr, unsigned int lower, unsigned int upper)
 {
-	MESSAGE_LIST_LOCK(ml, hide_lock);
-
+	MESSAGE_LIST_LOCK (ml, hide_lock);
+	
 	if (lower != ML_HIDE_SAME)
 		ml->hide_before = lower;
 	if (upper != ML_HIDE_SAME)
 		ml->hide_after = upper;
-
-	MESSAGE_LIST_UNLOCK(ml, hide_lock);
-
-	mail_regen_list(ml, ml->search, expr, NULL);
+	
+	MESSAGE_LIST_UNLOCK (ml, hide_lock);
+	
+	mail_regen_list (ml, ml->search, expr, NULL);
 }
 
 /* hide specific uid's */
 void
-message_list_hide_uids(MessageList *ml, GPtrArray *uids)
+message_list_hide_uids (MessageList *ml, GPtrArray *uids)
 {
 	int i;
 	char *uid;
 
 	/* first see if we need to do any work, if so, then do it all at once */
-	for (i=0;i<uids->len;i++) {
-		if (g_hash_table_lookup(ml->uid_nodemap, uids->pdata[i])) {
-			MESSAGE_LIST_LOCK(ml, hide_lock);
+	for (i = 0; i < uids->len; i++) {
+		if (g_hash_table_lookup (ml->uid_nodemap, uids->pdata[i])) {
+			MESSAGE_LIST_LOCK (ml, hide_lock);
 			if (ml->hidden == NULL) {
-				ml->hidden = g_hash_table_new(g_str_hash, g_str_equal);
-				ml->hidden_pool = e_mempool_new(512, 256, E_MEMPOOL_ALIGN_BYTE);
+				ml->hidden = g_hash_table_new (g_str_hash, g_str_equal);
+				ml->hidden_pool = e_mempool_new (512, 256, E_MEMPOOL_ALIGN_BYTE);
 			}
-	
-			uid =  e_mempool_strdup(ml->hidden_pool, uids->pdata[i]);
-			g_hash_table_insert(ml->hidden, uid, uid);
-			for (;i<uids->len;i++) {
-				if (g_hash_table_lookup(ml->uid_nodemap, uids->pdata[i])) {
-					uid =  e_mempool_strdup(ml->hidden_pool, uids->pdata[i]);
-					g_hash_table_insert(ml->hidden, uid, uid);
+			
+			uid =  e_mempool_strdup (ml->hidden_pool, uids->pdata[i]);
+			g_hash_table_insert (ml->hidden, uid, uid);
+			for ( ; i < uids->len; i++) {
+				if (g_hash_table_lookup (ml->uid_nodemap, uids->pdata[i])) {
+					uid =  e_mempool_strdup (ml->hidden_pool, uids->pdata[i]);
+					g_hash_table_insert (ml->hidden, uid, uid);
 				}
 			}
-			MESSAGE_LIST_UNLOCK(ml, hide_lock);
-			mail_regen_list(ml, ml->search, NULL, NULL);
+			MESSAGE_LIST_UNLOCK (ml, hide_lock);
+			mail_regen_list (ml, ml->search, NULL, NULL);
 			break;
 		}
 	}
@@ -2160,18 +2210,18 @@ message_list_hide_uids(MessageList *ml, GPtrArray *uids)
 void
 message_list_hide_clear (MessageList *ml)
 {
-	MESSAGE_LIST_LOCK(ml, hide_lock);
+	MESSAGE_LIST_LOCK (ml, hide_lock);
 	if (ml->hidden) {
-		g_hash_table_destroy(ml->hidden);
-		e_mempool_destroy(ml->hidden_pool);
+		g_hash_table_destroy (ml->hidden);
+		e_mempool_destroy (ml->hidden_pool);
 		ml->hidden = NULL;
 		ml->hidden_pool = NULL;
 	}
 	ml->hide_before = ML_HIDE_NONE_START;
 	ml->hide_after = ML_HIDE_NONE_END;
-	MESSAGE_LIST_UNLOCK(ml, hide_lock);
-
-	mail_regen_list(ml, ml->search, NULL, NULL);
+	MESSAGE_LIST_UNLOCK (ml, hide_lock);
+	
+	mail_regen_list (ml, ml->search, NULL, NULL);
 }
 
 #define HIDE_STATE_VERSION (1)
@@ -2277,6 +2327,13 @@ struct _regen_list_msg {
   use vfolder to implement searches ???
 
  */
+
+static char *
+regen_list_describe (struct _mail_msg *mm, gint complete)
+{
+	return g_strdup (_("Generating message list"));
+}
+
 static void
 regen_list_regen (struct _mail_msg *mm)
 {
@@ -2284,7 +2341,7 @@ regen_list_regen (struct _mail_msg *mm)
 	GPtrArray *uids, *uidnew, *showuids;
 	CamelMessageInfo *info;
 	int i;
-	
+
 	if (m->search)
 		uids = camel_folder_search_by_expression (m->folder, m->search, &mm->ex);
 	else
@@ -2400,6 +2457,9 @@ static void
 regen_list_regened (struct _mail_msg *mm)
 {
 	struct _regen_list_msg *m = (struct _regen_list_msg *)mm;
+
+	if (GTK_OBJECT_DESTROYED(m->ml))
+		return;
 	
 	if (m->summary == NULL)
 		return;
@@ -2408,7 +2468,7 @@ regen_list_regened (struct _mail_msg *mm)
 		build_tree (m->ml, m->tree, m->changes);
 	else
 		build_flat (m->ml, m->summary, m->changes);
-	
+
 	gtk_signal_emit (GTK_OBJECT (m->ml), message_list_signals[MESSAGE_LIST_BUILT]);
 }
 
@@ -2442,7 +2502,7 @@ regen_list_free (struct _mail_msg *mm)
 }
 
 static struct _mail_msg_op regen_list_op = {
-	NULL,
+	regen_list_describe,
 	regen_list_regen,
 	regen_list_regened,
 	regen_list_free,
@@ -2458,7 +2518,7 @@ mail_regen_list (MessageList *ml, const char *search, const char *hideexpr, Came
 	
 #ifndef BROKEN_ETREE
 	/* this can sometimes crash,so ... */
-
+	
 	/* see if we need to goto the child thread at all anyway */
 	/* currently the only case is the flat view with updates and no search */
 	if (hideexpr == NULL && search == NULL && changes != NULL && !ml->threaded) {
