@@ -64,7 +64,8 @@ make_initial_rdf_list (ESummaryPrefs *prefs)
 {
 	GList *rdfs;
 
-	rdfs = g_list_prepend (NULL, g_strdup ("http://www.cnn.com/cnn.rss"));
+	rdfs = g_list_prepend (NULL, g_strdup ("http://linuxtoday.com/backend/my-netscape.rdf"));
+	rdfs = g_list_append (rdfs, g_strdup ("http://www.salon.com/feed/RDF/salon_use.rdf"));
 	
 	prefs->rdf_urls = rdfs;
 }
@@ -460,7 +461,6 @@ static struct _RDFInfo rdfs[] = {
 	{"http://www.fool.com/about/headlines/rss_headlines.asp", "The Motley Fool"},
 	{"http://www.newsforge.com/newsforge.rss", "Newsforge"},
 	{"http://www.nanotechnews.com/nano/rdf", "Nanotech News"},
-  	{"http://www.perl.com/pace/news.rss", "Perl.com"},
 	{"http://www.pigdog.org/pigdog.rdf", "Pigdog"},
 	{"http://www.python.org/channews.rdf", "Python.org"},
 	{"http://www.quotationspage.com/data/mqotd.rss", N_("Quotes of the Day")},
