@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,12 +25,17 @@
    <?xml version="1.0"?>
    <efolder>
            <type>mail</type>
-	   <description>This is the folder where I store mail from my gf</description>
+
+	   <name>Inbox</name>
+	   <name locale="it">Posta in Arrivo</name>
+
+	   <description>This is the default folder for incoming messages</description>
+	   <description locale="it">Cartella che contiene i messaggi in arrivo</description>
+
 	   <homepage>http://www.somewhere.net</homepage>
    </efolder>
 
    FIXME: Do we want to use a namespace for this?
-   FIXME: Do we want to have an internationalized description?
  */
 
 #ifdef HAVE_CONFIG_H
@@ -327,8 +331,8 @@ save_metadata (ELocalFolder *local_folder)
 		     (xmlChar *) e_folder_get_type_string (folder));
 
 	if (e_folder_get_description (folder) != NULL)
-		xmlNewChild (root, NULL, (xmlChar *) "description",
-			     (xmlChar *) e_folder_get_description (folder));
+		xmlNewTextChild (root, NULL, (xmlChar *) "description",
+				 (xmlChar *) e_folder_get_description (folder));
 
 	physical_directory = e_folder_get_physical_uri (folder) + URI_PREFIX_LEN - 1;
 	physical_path = g_concat_dir_and_file (physical_directory, E_LOCAL_FOLDER_METADATA_FILE_NAME);
