@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,10 @@
 
 #include <glib.h>
 
-char *e_shell_get_icon_path (const char *icon_name,
-			     gboolean try_mini);
+char     *e_shell_get_icon_path         (const char  *icon_name,
+					 gboolean     try_mini);
+
+gboolean  e_shell_folder_name_is_valid  (const char  *name,
+					 const char **reason_return);
 
 #endif
